from optparse import OptionParser
import sys
from .debug import simple as debug_simple

opt_configs = [
{
    "short": "p",
    "long": "projectname",
    "dest": "projectname",
    "help": "the project name"
},
    {
        "short": "c",
        "long": "cfg",
        "dest": "cfg",
        "help": "the banyan configuration file name",
        "default": "dev"
    },
    {
        "short": "u",
        "long": "giturl",
        "dest": "giturl",
        "help": "git url for the project",
        "default": None
    },
    {
        "short": "t",
        "long": "tag",
        "dest": "tag",
        "help": "tag of the project",
        "default": "v1.0.0"
    },
    {
        "short": "s",
        "long": "only-structure",
        "dest": "only_structure",
        "help": "only build the structure",
        "default": "False"
    },
    {
        "short": "k",
        "long": "key-file",
        "dest": "key_file",
        "help": "private key file for accessing the remote",
        "default": None
    },
    {
        "short": "r",
        "long": "remote-addr",
        "dest": "remote_addr",
        "help": "remote address",
        "default": None
    },
    {
        "short": "n",
        "long": "user-name",
        "dest": "user_name",
        "help": "user name of the remote",
        "default": None
    },
    {
        "short": "g",
        "long": "build-gate",
        "dest": "build_gate",
        "help": "default is false to not build the gate",
        "default": False
    },
    {
        "short": "o",
        "long": "role-tags",
        "dest": "role_tags",
        "help": "specify role tags to only run the roles",
        "default": None
    },
    {
        "short": "m",
        "long": "proxy-mapping",
        "dest": "proxy_mapping",
        "action": "append",
        "help": "specify the proxy mapping",
        "default": None
    },
    {
        "short": "a",
        "long": "noauth",
        "dest": "noauth",
        "action": "append",
        "help": "do not specify the auth for project",
        "default": None
    }
]
def get_options(sys_args = None):

    def configure_parser(parser):
        def add_opt(opt):
            parser.add_option("-"+opt["short"], \
                              "--" + opt["long"], \
                              dest=opt["dest"] if "dest" in opt else None, \
                              help=opt["help"] if "help" in opt else None, \
                              default = opt["default"] if "default" in opt else None, \
                              action = opt["action"] if "action" in opt else None
            )

        [add_opt(opt) for opt in opt_configs]
        
        (options, args) = parser.parse_args(debug_simple(sys_args or sys.argv, "original args"))

        class dumy:
            def __init__(self):
                self.projectname = options.projectname
                self.cfg = options.cfg
                self.command = args[1]
                self.giturl = options.giturl
                self.tag = options.tag
                self.only_structure = options.only_structure.strip().lower() == 'true'
                self.key_file = options.key_file
                self.remote_addr = options.remote_addr
                self.user_name = options.user_name
                self.build_gate = options.build_gate == True or (isinstance(options.build_gate, str) and  options.build_gate.lower() == "true")
                self.role_tags = None if options.role_tags == None else options.role_tags.split(',')
                self.proxy_mapping = options.proxy_mapping
                self.noauth = options.noauth
                
        return dumy()

    return configure_parser(OptionParser())



import os

debug_flag = "BANYAN_DEBUG"
def simple(obj, title = None):
    if os.environ.get(debug_flag):
        if title:
            print("%s:%s"%(title, obj))
        else:
            print(obj)
    return obj
    

def on():
    os.environ[debug_flag] = "debug"

def off():
    os.environ[debug_flag] = None



import os
from .githelper import get_tag, get_project_name_from_url
from .config import load_all_dependencies
import demjson

def init_tag_for_project(giturl, tag, config_name, location=os.getcwd(), key_file_path = None):
    get_tag(giturl, tag, location, key_file_path)

    def load_tag(dep_config):
        get_tag(dep_config["giturl"], dep_config["tag"], location, key_file_path)
        
        return demjson.decode_file(os.path.join(location, get_project_name_from_url(dep_config["giturl"]), "deploy", dep_config["config_name"] + ".cfg"))
    
    load_all_dependencies(os.path.join(location, get_project_name_from_url(giturl), "deploy", config_name + ".cfg"), load_tag)



import os
from .folder import put_folder

def get_file_name(path):
    return os.path.split(path)[1]

def get_file_only_name(path):
    return get_file_name(path).split('.')[0]

def put_file(path):
    '''
make sure the folder of the file exists
'''
    return os.path.join(put_folder(os.path.dirname(path)), \
                        get_file_name(path) \
                        )

        
        



import os

def put_folder(path):
    '''
if the path of the folder doesn't exist, it will create the folder.
the function will return the path
'''
    if not os.path.exists(path):
        os.makedirs(path)
    return path



from .util import get_config_path, get_project_path
from fn import Stream
import os

def init_config(project_name, config_name):
    def get_role_names():
        return ",".join(['"' + dir + '"' for dir in os.listdir(os.path.join(get_project_path(project_name), "deploy", "roles"))])
    
    def gen_content():return '''{{
    "project_name": "{project_name}",
    "dependencies": [{{
        "project_name": "<project_name>",
        "git": "<git_url>",
        "config_name": "<configuration_name>"
    }}
    ],
    "roles_seq": [{roles_arr}],
    "predefined_variables": {{}}
}}
'''.format(project_name=project_name, roles_arr=get_role_names())

    open(get_config_path(project_name, config_name), 'w').write(gen_content())



from .banyan_opt import get_options
import sys
import os
from .deploy import build_deploy_script, run_deploy, deploy_config
import logging
logging.basicConfig(stream = sys.stdout, level = 'DEBUG')
from .util import init_root, get_deploy_path

init_root()

def run(options):
    def root_folder():
        return os.getcwd()
    
    def banyan_config_path():
        return os.path.join(root_folder(), options.projectname, "deploy", "dev.cfg" if options.cfg == None else options.cfg + ".cfg")

        
    def error_handler(msg):
        print(msg)

    def get_remote_dict():
        return {"key": options.key_file, "remote_user": options.user_name, "remote_addr": options.remote_addr} if \
            not (options.key_file == None or options.user_name == None or options.remote_addr == None) else None
        
    if options.command == "deploy":
        run_deploy(options.projectname, options.cfg, options.role_tags)
    elif options.command == "build":
        build_deploy_script(options.projectname, options.cfg, only_structure = options.only_structure, remote_dict = get_remote_dict(), gate = options.build_gate, proxy_mapping = options.proxy_mapping, noauth = options.noauth)
    elif options.command == "config":
        deploy_config(options.projectname, options.cfg)
    elif options.command == "get":
        from .get_code_with_tag import init_tag_for_project
        init_tag_for_project(options.giturl, options.tag, options.cfg, os.getcwd(), options.key_file)
    elif options.command == "init":
        from .init_config import init_config
        init_config(options.projectname, options.cfg)
    elif options.command == "clear":
        from .clear_deploy import clear_deploy
        clear_deploy(os.path.join(os.getcwd(), options.projectname), options.cfg)
    else:
        print('please set the valid command: deploy, init')
        
def main():
    run(get_options())



import os
import yaml
import functools
from .debug import simple
from jinja2 import Template
from fn import F

def all(parent_folder, role_names = []):
    def get_role_names(roles_path):
        def get_roles():
            return [y for x in [dirs for root, dirs, _ in os.walk(roles_path, followlinks = True) if root == roles_path] for y in x]

        return get_roles() if os.path.exists(roles_path) else []
    
    def get_defaults_by_name(rolename):
        def get_defaults(path):
            return yaml.load(open(path, 'r')) if os.path.exists(path) else {}

        return (F(lambda title, content: simple(content, title), "get_defaults_by_name_path") >> \
                F(get_defaults) >> \
                F(lambda title, content: simple(content, title), "get_defaults_by_name") >> \
                F(init_root, rolename) >> \
                F(lambda title, content: simple(content, title), "init_root") \
                )(os.path.join(parent_folder, "roles", rolename, "defaults", "main.yml")) \
                if len(role_names) == 0 or rolename in role_names else {}
                
    def combine(names):
        return functools.reduce(lambda a, b: {**a, **b}, names)
    return combine([get_defaults_by_name(rolename) for rolename in get_role_names(os.path.join(parent_folder, "roles"))])

def init_root(role_name, default_values):
    def get_project_name():
        return None if len(role_name.split('_')) < 2 else (lambda elements:"_".join(elements[0:len(elements)-1]))(role_name.split('_'))

    def init_root(key, value):
        return value if not key.endswith('_root') else os.path.join(os.getcwd(), get_project_name())
    
    assert get_project_name() != None, "role_name:%s is not in format like <project_name>_<role_name>" % role_name

    def write_root(new_values):
        for key in default_values:
            new_values[key] = init_root(key, default_values[key])

        return new_values
    
    return write_root({})

def write(file_path, keyvals, remote_addr = None):
    from .tools.data_convert import dict2assignments

    def get_remote_addr(): return '' if remote_addr == None else '[remote]' + os.linesep + remote_addr
    
    file = open(file_path, 'w')
    try:
        file.write(Template('''[all:vars]
{% for assignment in assignments %}{{assignment}}
{% endfor %}

{{remote_addr}}
''').render(assignments = dict2assignments(keyvals), remote_addr = get_remote_addr()))
        
    finally:
        file.close()

def override_values(cfg_json, host_values):
    def override(new_values):
        simple(host_values, "override_values_host_values").update(new_values)
        return host_values

    return override(cfg_json["predefined_variables"]) \
        if "predefined_variables" in cfg_json \
           else simple(host_values, "override_values_host_values")
        

